package org.expeditee.items.widgets.charts;

import java.awt.Font;

import javax.swing.JPanel;

import org.expeditee.gui.AttributeUtils;
import org.expeditee.gui.AttributeValuePair;
import org.expeditee.gui.Frame;
import org.expeditee.items.Text;
import org.expeditee.items.widgets.InteractiveWidget;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.title.LegendTitle;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.category.DefaultCategoryDataset;

public abstract class AbstractCategory extends InteractiveWidget {

	protected static final String DEFAULT_TITLE = "";

	protected DefaultCategoryDataset _data = new DefaultCategoryDataset();

	private JFreeChart _chart = null;

	private LegendTitle _legend = null;

	protected JFreeChart getChart() {
		return _chart;
	}

	protected DefaultCategoryDataset getData() {
		return _data;
	}

	public AbstractCategory(Text source, String[] args) {
		super(source, new JPanel(), 100, 300, -1, 100, 300, -1);

		init();
		refresh();
	}

	@Override
	protected String[] getArgs() {
		return null;
	}

	protected void init() {
		// create a chart...
		_chart = createNewChart();
		_chart.getPlot().setNoDataMessage("Add link to data frame");
		_legend = _chart.getLegend();
		ChartPanel cp = new ChartPanel(_chart);
		cp.setPopupMenu(null);
		super._swingComponent = cp;
	}

	protected abstract JFreeChart createNewChart();

	@Override
	public void refresh() {
		super.refresh();
		// Get the data from the linked frame
		Frame dataFrame = getDataFrame();

		_data.clear();
		_chart.clearSubtitles();

		if (dataFrame != null) {
			_chart.setTitle(dataFrame.getTitle());

			for (Text t : dataFrame.getBodyTextItems(false)) {
				AttributeValuePair avp = AttributeUtils.getPair(t.getText());
				if (avp != null) {
					_data
							.setValue(avp.getDoubleValue(), "", avp
									.getAttribute());
				}
			}

			if (dataFrame.hasAnnotation("legend")) {
				_chart.addLegend(_legend);
			}
			if (dataFrame.hasAnnotation("subtitle")) {
				getChart().addSubtitle(
						new TextTitle(dataFrame.getAnnotationValue("subtitle"),
								JFreeChart.DEFAULT_TITLE_FONT.deriveFont(
										Font.ITALIC,
										JFreeChart.DEFAULT_TITLE_FONT
												.getSize2D() * .7F)));
			}
			CategoryPlot plot = (CategoryPlot) getChart().getPlot();
			if (dataFrame.hasAnnotation("xaxis")) {
				plot.getRangeAxis().setLabel(
						dataFrame.getAnnotationValue("xaxis"));
			}
			if (dataFrame.hasAnnotation("yaxis")) {
				plot.getDomainAxis().setLabel(
						dataFrame.getAnnotationValue("yaxis"));
			}
			if (dataFrame.hasAnnotation("horizontal")) {
				plot.setOrientation(PlotOrientation.HORIZONTAL);
			} else {
				plot.setOrientation(PlotOrientation.VERTICAL);
			}

		} else {
			_chart.setTitle(this.getClass().getSimpleName() + " Chart");
		}
		super._swingComponent.invalidate();
	}

	@Override
	public float getMinimumBorderThickness() {
		return 1.0F;
	}
}
