package org.expeditee.items.widgets.charts;

import java.awt.Font;

import javax.swing.JPanel;

import org.expeditee.gui.AttributeUtils;
import org.expeditee.gui.AttributeValuePair;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.items.Item;
import org.expeditee.items.Text;
import org.expeditee.items.widgets.InteractiveWidget;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.PieSectionLabelGenerator;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.title.LegendTitle;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.general.DefaultPieDataset;

public abstract class AbstractPie extends InteractiveWidget {

	protected static final String DEFAULT_TITLE = "";

	protected DefaultPieDataset _data = new DefaultPieDataset();

	private JFreeChart _chart = null;

	private LegendTitle _legend = null;

	private PieSectionLabelGenerator _labelGenerator;

	protected JFreeChart getChart() {
		return _chart;
	}

	protected DefaultPieDataset getData() {
		return _data;
	}

	public AbstractPie(Text source, String[] args) {
		super(source, new JPanel(), 100, 300, -1, 100, 300, -1);

		init();
		refresh();
	}

	@Override
	protected String[] getArgs() {
		return null;
	}

	protected void init() {
		// create a chart...
		_chart = createNewChart();
		_chart.getPlot().setNoDataMessage("Add link to data frame");
		_legend = _chart.getLegend();
		_labelGenerator = ((PiePlot) getChart().getPlot()).getLabelGenerator();
		ChartPanel cp = new ChartPanel(_chart);
		cp.setPopupMenu(null);
		super._swingComponent = cp;
	}

	protected abstract JFreeChart createNewChart();

	@Override
	public void refresh() {
		super.refresh();
		// Get the data from the linked frame
		Frame dataFrame = getDataFrame();

		_data.clear();
		_chart.clearSubtitles();

		if (dataFrame != null) {
			_chart.setTitle(dataFrame.getTitle());

			for (Text t : dataFrame.getBodyTextItems(false)) {
				AttributeValuePair avp = AttributeUtils.getPair(t.getText());
				if (avp != null) {
					_data.setValue(avp.getAttribute(), avp.getDoubleValue());
				}
			}

			if (dataFrame.hasAnnotation("legend")) {
				_chart.addLegend(_legend);
			}
			if (dataFrame.hasAnnotation("subtitle")) {
				getChart().addSubtitle(
						new TextTitle(dataFrame.getAnnotationValue("subtitle"),
								JFreeChart.DEFAULT_TITLE_FONT.deriveFont(
										Font.ITALIC,
										JFreeChart.DEFAULT_TITLE_FONT
												.getSize2D() * .7F)));
			}
			if (dataFrame != null) {
				if (dataFrame.hasAnnotation("labels")) {
					((PiePlot) getChart().getPlot())
							.setLabelGenerator(_labelGenerator);
				} else {
					((PiePlot) getChart().getPlot()).setLabelGenerator(null);
				}
			}
		} else {
			_chart.setTitle(this.getClass().getSimpleName() + " Chart");
		}
		super._swingComponent.invalidate();
	}

	@Override
	public float getMinimumBorderThickness() {
		return 1.0F;
	}
}
